import os
import json
import tempfile
import playwright
import patchright
from dektools.file import replace_file
from dektools.web.js.polyfill import btoa


class CodeGen:
    def __init__(self):
        self._path = tempfile.mktemp(prefix='playwright-codegen')
        self._last_mtime = None
        self._content = b''

    def __del__(self):
        if os.path.isfile(self._path):
            os.remove(self._path)

    def init(self, lang='jsonl', **kwargs):
        os.environ['PLAYWRIGHT_RECODER_PARAMS'] = btoa(
            json.dumps({'language': lang, 'outputFile': self._path, **kwargs}))

    async def update(self, changed=None):
        if os.path.isfile(self._path):
            mt = os.path.getmtime(self._path)
            if mt != self._last_mtime:
                with open(self._path, 'rb') as f:
                    f.seek(len(self._content))
                    delta = f.read()
                    self._content += delta
                    if changed:
                        await changed(delta)
                self._last_mtime = mt

    @property
    def content(self):
        return self._content

    @property
    def text(self):
        return self._content.decode('utf-8')

    @staticmethod
    def fix_package(reverse=False):
        replace_file({
            os.path.join(
                os.path.dirname(os.path.abspath(module.__file__)),
                'driver/package/lib/server/recorder/contextRecorder.js'
            ): [
                [
                    ' params, delegate) {',
                    """params, delegate) {
let paramsData = {}
let paramsEnv = process.env.PLAYWRIGHT_RECODER_PARAMS
if(paramsEnv) paramsData = JSON.parse(Buffer.from(paramsEnv, 'base64').toString('ascii'))
params = Object.assign({}, params, paramsData)"""
                ]
            ] for module in (playwright, patchright)
        }, reverse)
